@@ -3,17 +3,17 @@ package BarWithRequires;
 use MooX::Role::Parameterized;
 
 role {
-    my ( $params, $p ) = @_;
+    my ( $params, $mop ) = @_;
 
-    $p->has( $params->{attr} => ( is => 'rw' ) );
+    $mop->has( $params->{attr} => ( is => 'rw' ) );
 
-    $p->method(
+    $mop->method(
         $params->{method} => sub {
             1024;
         }
     );
 
-    $p->requires( $params->{requires} );
+    $mop->requires( $params->{requires} );
 };
 
 use Moo::Role;
